package cn.com.qmth.scancentral.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.scancentral.dao.ExamRoomDao;
import cn.com.qmth.scancentral.entity.ExamRoomEntity;
import cn.com.qmth.scancentral.service.ExamRoomService;
import cn.com.qmth.scancentral.util.PageUtil;
import cn.com.qmth.scancentral.vo.CampusVo;
import cn.com.qmth.scancentral.vo.ExamSiteVo;
import cn.com.qmth.scancentral.vo.examroom.ExamRoomScannedQuery;
import cn.com.qmth.scancentral.vo.examroom.ExamRoomScannedVo;

@Service
public class ExamRoomServiceImpl extends ServiceImpl<ExamRoomDao, ExamRoomEntity> implements ExamRoomService {

    @Override
    public PageResult<ExamRoomScannedVo> examRoomScannedPage(ExamRoomScannedQuery query) {
        IPage<ExamRoomScannedVo> iPage = baseMapper
                .examRoomScannedPage(new Page<>(query.getPageNumber(), query.getPageSize()), query);
        for (ExamRoomScannedVo vo : iPage.getRecords()) {
            if (!vo.getScanned()) {
                vo.setScannedText("未扫描");
            } else {
                vo.setScannedText("已扫描");
            }
        }
        return PageUtil.of(iPage);
    }

    @Transactional
    @Override
    public void clearByExamAndSubject(Long examId, String subjectCode) {
        QueryWrapper<ExamRoomEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamRoomEntity> lw = wrapper.lambda();
        lw.eq(ExamRoomEntity::getExamId, examId);
        lw.eq(ExamRoomEntity::getSubjectCode, subjectCode);
        this.baseMapper.delete(wrapper);
    }

    @Transactional
    @Override
    public void modifyByExamAndSubject(Long examId, String subjectCode) {
        clearByExamAndSubject(examId, subjectCode);
        this.baseMapper.modifyByExamAndSubject(examId, subjectCode);
    }

    @Transactional
    @Override
    public void modifyByStudentScan(ExamRoomScannedQuery req) {
        QueryWrapper<ExamRoomEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamRoomEntity> lw = wrapper.lambda();
        lw.eq(ExamRoomEntity::getExamId, req.getExamId());
        lw.eq(ExamRoomEntity::getSubjectCode, req.getSubjectCode());
        lw.eq(ExamRoomEntity::getProvince, req.getProvince());
        lw.eq(ExamRoomEntity::getExamSite, req.getExamSite());
        lw.eq(ExamRoomEntity::getCampusCode, req.getCampusCode());
        lw.eq(ExamRoomEntity::getExamRoom, req.getExamRoom());
        ExamRoomEntity e = this.getOne(wrapper);
        if (!e.getScanned()) {
            e.setScanned(true);
            this.save(e);
        }
    }

    @Override
    public List<CampusVo> listCampusByExamId(Long examId) {
        return this.baseMapper.listCampusByExamId(examId);
    }

    @Override
    public List<ExamSiteVo> listExamSiteByExamId(Long examId) {
        return this.baseMapper.listExamSiteByExamId(examId);
    }

    @Override
    public List<String> listProvinceByExamId(Long examId) {
        return this.baseMapper.listProvinceByExamId(examId);
    }

    @Override
    public List<String> listExamRoomByExamId(Long examId) {
        return this.baseMapper.listExamRoomByExamId(examId);
    }

    @Override
    public List<ExamRoomScannedVo> examRoomScannedList(ExamRoomScannedQuery query) {
        return examRoomScannedPage(query).getResult();
    }
}
